'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FetchContext = exports.useFetch = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utils = require('./utils');

var _SimpleCache = require('./SimpleCache');

var _SimpleCache2 = _interopRequireDefault(_SimpleCache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var FetchContext = _react2.default.createContext({});

function getOptions(options) {
  return (0, _utils.isFunction)(options) ? options() : options;
}

function getCache(props) {
  return props.cache === true ? new _SimpleCache2.default() : (0, _utils.isObject)(props.cache) ? props.cache : null;
}

function useMountedRef() {
  var mountedRef = (0, _react.useRef)(true);
  (0, _react.useEffect)(function () {
    mountedRef.current = true;
    return function () {
      mountedRef.current = false;
    };
  }, []);

  return mountedRef;
}

function useFetch(props) {
  var _useState = (0, _react.useState)({
    request: {
      url: props.url,
      options: props.options
    },
    loading: props.manual ? null : true,
    data: undefined,
    error: undefined
  }),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  state.fetch = doFetch;
  state.clearData = function () {
    return setState(_extends({}, state, { data: undefined }));
  };

  var cache = (0, _react.useRef)(getCache(props)); // TODO: Update on cache prop change
  var promises = (0, _react.useRef)([]);
  var mountedRef = useMountedRef();

  var as = props.as || 'auto';
  var fetchFunction = props.fetchFunction || function (url, options) {
    return fetch(url, options);
  };

  (0, _react.useEffect)(function () {
    if ((0, _utils.isFunction)(props.onChange)) {
      props.onChange(state);
    }

    if (props.url && !props.manual) {
      doFetch(props.url, props.options);
    }
  }, [props.url, props.manual].concat(_toConsumableArray(props.deps || [])));

  function doFetch(url, options, updateOptions) {
    if (url == null) {
      url = props.url;
    }

    options = getOptions(options || props.options);
    var request = { url: url, options: options };

    if (cache.current && cache.current.get(url)) {
      // Restore cached state
      var promise = cache.current.get(url);
      promise.then(function (cachedState) {
        return update(cachedState, promise, updateOptions);
      });
      promises.current.push(promise);
    } else {
      update({ request: request, loading: true }, null, updateOptions);

      var _promise = fetchFunction(url, options).then(function (response) {
        var dataPromise = (0, _utils.isFunction)(as) ? as(response) : (0, _utils.isObject)(as) ? (0, _utils.parseBody)(response, as) : as === 'auto' ? (0, _utils.parseBody)(response) : response[as]();

        return dataPromise.then(function (data) {
          return { response: response, data: data };
        }).catch(function (error) {
          return { response: response, data: error };
        });
      }).then(function (_ref) {
        var _newState;

        var response = _ref.response,
            data = _ref.data;

        var newState = (_newState = {
          request: request,
          loading: false
        }, _defineProperty(_newState, response.ok ? 'error' : 'data', undefined), _defineProperty(_newState, response.ok ? 'data' : 'error', data), _defineProperty(_newState, 'response', response), _newState);

        update(newState, _promise, updateOptions);

        return newState;
      }).catch(function (error) {
        // Catch request errors with no response (CORS issues, etc)
        var newState = {
          request: request,
          data: undefined,
          error: error,
          loading: false
        };

        update(newState, _promise, updateOptions);

        // Rethrow so not to swallow errors, especially from errors within handlers (children func / onChange)
        throw error;

        return newState;
      });

      promises.current.push(_promise);

      if (cache.current) {
        cache.current.set(url, _promise);
      }

      return _promise;
    }
  }

  function update(nextState, currentPromise) {
    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    if (currentPromise) {
      // Handle (i.e. ignore) promises resolved out of order from requests
      var index = promises.current.indexOf(currentPromise);
      if (index === -1) {
        // Ignore update as a later request/promise has already been processed
        return;
      }

      // Remove currently resolved promise and any outstanding promises
      // (which will cause them to be ignored when they do resolve/reject)
      promises.current.splice(0, index + 1);
    }

    var onChange = props.onChange,
        onDataChange = props.onDataChange;


    var data = undefined;
    if (nextState.data && nextState.data !== state.data && (0, _utils.isFunction)(onDataChange)) {
      data = onDataChange(nextState.data, options.ignorePreviousData ? undefined : state.data);
    }

    if ((0, _utils.isFunction)(onChange)) {
      // Always call onChange even if unmounted.  Useful for `POST` requests with a redirect
      onChange(_extends({}, state, nextState, data !== undefined && { data: data }));
    }

    // Ignore passing state down if no longer mounted
    if (mountedRef.current) {
      // If `onDataChange` prop returned a value, we use it for data passed down to the children function
      setState(_extends({}, state, nextState, data !== undefined && { data: data }));
    }
  }

  return state;
}

var Fetch = function Fetch(_ref2) {
  var children = _ref2.children,
      props = _objectWithoutProperties(_ref2, ['children']);

  return _react2.default.createElement(
    FetchContext.Provider,
    { value: useFetch(props) },
    (0, _utils.isFunction)(children) ? _react2.default.createElement(
      FetchContext.Consumer,
      null,
      children
    ) : children
  );
};
Fetch.Consumer = FetchContext.Consumer;

exports.useFetch = useFetch;
exports.FetchContext = FetchContext;
exports.default = Fetch;